@@ -124,7 +124,7 @@ class StateRequest(Message):
 
 
 class StateResponse(Message):
-    __slots__ = ('state',)
+    __slots__ = ('state', 'compute_metrics')
 
 
 class GetParametersRequest(Message):
@@ -164,7 +164,7 @@ class ConnectRequest(Message):
 
 
 class ConnectResponse(Message):
-    __slots__ = ('state',)
+    __slots__ = ('state', 'compute_metrics')
 
 
 class PushGraphsRequest(Message):
@@ -184,4 +184,8 @@ class ReplayRequest(Message):
 
 
 class ReplayResponse(Message):
-    __slots__ = ('enable',)
\ No newline at end of file
+    __slots__ = ('enable',)
+
+
+class SetComputeMetricsRequest(Message):
+    __slots__ = ('compute_metrics',)
\ No newline at end of file
