@@ -1,7 +1,10 @@
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from enum import Enum, auto
+from typing import ClassVar
+from uuid import uuid4
 
 from qtpy.QtWidgets import QWidget
+from qtpy.QtCore import Signal, QObject
 
 
 class Location(Enum):
@@ -21,13 +24,24 @@ class RenderMode(Enum):
     Background = auto()
 
 
+class GraphSignalRelay(QObject):
+    sigPush = Signal(object)
+
+
+graph_signal_relay = GraphSignalRelay()
+
+
 @dataclass(eq=False)
 class Graph:
-    name: str
+    name: ClassVar[str] = ''
     compute_with: Location = Location.Client
     compute_mode: ComputeMode = ComputeMode.Blocking
     render_mode: RenderMode = RenderMode.Blocking
-    widget: QWidget = None
+    widget_class: ClassVar[type[QWidget]] = None
+    widget_args: tuple = field(default_factory=tuple)
+    widget_kwargs: dict = field(default_factory=dict)
+    id: str = field(default_factory=lambda: str(uuid4()))
+
 
     def compute(self, data: 'Data', *args):
         ...
@@ -36,7 +50,7 @@ class Graph:
         ...
 
     def make_widget(self):
-        ...
+        return self.widget_class(*self.widget_args, **self.widget_kwargs)
 
     # @property
     # def widget(self):
