@@ -14,36 +14,6 @@ from qtpy.QtCore import QObject, Signal
 from tsuchinoko.utils import threads
 
 
-def _get_asyncio_queue(loop):
-    class AsyncioQueue(asyncio.Queue):
-        '''
-        Asyncio queue modified for caproto server layer queue API compatibility
-
-        NOTE: This is bound to a single event loop for compatibility with
-        synchronous requests.
-        '''
-
-        def __init__(self, *, loop=loop, **kwargs):
-            super().__init__(loop=loop, **kwargs)
-
-        async def async_get(self):
-            return await super().get()
-
-        async def async_put(self, value):
-            return await super().put(value)
-
-        def get(self):
-            future = asyncio.run_coroutine_threadsafe(self.async_get(), loop)
-            return future.result()
-
-        def put(self, value):
-            future = asyncio.run_coroutine_threadsafe(
-                self.async_put(value), loop)
-            return future.result()
-
-    return AsyncioQueue
-
-
 @dataclass(order=True)
 class PrioritizedPlan:
     priority: int
@@ -183,6 +153,8 @@ class QRunEngine(QObject):
 
     def _put(self, priority, args, kwargs):
         self.queue.put(PrioritizedPlan(priority, (args, kwargs)))
+        if not self.process_queue_thread.running:
+            self.process_queue_thread.start()
 
     def _check_if_ready(self):
         # RE has finished processing everything in the queue
