@@ -1,16 +1,10 @@
-import os
-import re
-import tempfile
-from pathlib import Path
+
 from time import perf_counter
 
 import numpy as np
-from PySide2.QtCore import Qt, QTimer
-from PySide2.QtWidgets import QAction, QFileDialog
+from qtpy.QtCore import Qt, QTimer
 from loguru import logger
-from pyqtgraph.exporters import ImageExporter
-from qtpy import QtWidgets
-from qtpy.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout
+from qtpy.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QComboBox
 from pyqtgraph import InfiniteLine, mkPen, PlotWidget, HistogramLUTWidget, mkBrush, functions as fn, FileDialog
 
 from tsuchinoko.graphics_items.clouditem import CloudItem
@@ -19,6 +13,7 @@ from tsuchinoko.graphics_items.mixins import ClickRequesterPlot
 
 last_export_directory = None
 
+
 class CloudWidget(QWidget):
     noRepeatKeys = [
         Qt.Key.Key_Right,
@@ -30,10 +25,10 @@ class CloudWidget(QWidget):
     ]
 
     def __init__(self, data_key:str, accumulates:bool):
+        self.graph = ClickRequesterPlot()
         super().__init__()
         self.data_key = data_key
         self.accumulates = accumulates
-        graph = ClickRequesterPlot()
         # scatter = ScatterPlotItem(name='scatter', x=[0], y=[0], size=10, pen=mkPen(None), brush=mkBrush(255, 255, 255, 120))
         self.timeline = InfiniteLine(0, pen=mkPen(width=3), movable=True)
         self.timeline.sigPositionChanged.connect(self.timeline_changed)
@@ -45,16 +40,27 @@ class CloudWidget(QWidget):
         self.cloud = CloudItem(name='scatter', size=10)
         histlut = HistogramLUTWidget()
         histlut.setImageItem(self.cloud)
+        self.output_selector = QComboBox()
+        self.output_selector.setHidden(True)
+        self.output_selector.currentIndexChanged.connect(self.invalidate_cloud)
 
         self.setLayout(QVBoxLayout())
+        self.layout().setContentsMargins(0,0,0,0)
+        self.layout().setSpacing(0)
         hlayout = QHBoxLayout()
         self.layout().addLayout(hlayout)
+        right_layout = QVBoxLayout()
+        right_layout.setSpacing(0)
+        right_layout.setContentsMargins(0,0,0,0)
+
+
         self.layout().addWidget(self.timeline_plot)
+        hlayout.addWidget(self.graph)
+        hlayout.addLayout(right_layout)
+        right_layout.addWidget(histlut)
+        right_layout.addWidget(self.output_selector)
 
-        hlayout.addWidget(graph)
-        hlayout.addWidget(histlut)
-
-        graph.addItem(self.cloud)
+        self.graph.addItem(self.cloud)
 
         # Hard-coded to show max
         self.max_arrow = BetterCurveArrow(self.cloud.scatter, brush=mkBrush('r'))
@@ -71,6 +77,9 @@ class CloudWidget(QWidget):
         self.last_play_time = 0
         self.play_timer.timeout.connect(self.timeout)
 
+    def invalidate_cloud(self):
+        self.cloud.clear()
+
     def timeline_changed(self):
         if not self.cache:
             return
@@ -200,10 +209,24 @@ class CloudWidget(QWidget):
         timeline_at_end = not self.cache or self.timeline.getXPos() == len(self.cache['v']) - 1
 
         with data.r_lock():
-            v = data[self.data_key].copy()
-
+            v = np.asarray(data[self.data_key].copy())
             x, y = zip(*data.positions)
 
+        if v.ndim == 2:
+            if len(v[0]) > 1:
+                self.output_selector.setHidden(False)
+                if self.output_selector.count() != len(v[0]):
+                    self.output_selector.clear()
+                    for i in range(len(v[0])):
+                        self.output_selector.addItem(f'{i}')
+                    self.output_selector.setCurrentIndex(0)
+                v = v[:, self.output_selector.currentIndex()]
+            else:
+                try:
+                    v = np.squeeze(v, 1)
+                except ValueError:
+                    pass
+
         lengths = len(v), len(x), len(y)
         min_length = min(lengths)
 
@@ -264,3 +287,6 @@ class CloudWidget(QWidget):
 
         # text.setText(f'Max: {v[max_index]:.2f} ({x[max_index]:.2f}, {y[max_index]:.2f})')
         # text.setPos(x[max_index], y[max_index])
+
+    def getView(self):
+        return self.graph
\ No newline at end of file
