@@ -1,6 +1,7 @@
 import time
 from collections import defaultdict
 from functools import partial
+from pickle import UnpicklingError
 from queue import Queue, Empty
 import subprocess
 from signal import SIGINT
@@ -10,6 +11,7 @@ from pathlib import Path
 
 from tsuchinoko.utils.dependencies import check_dependencies
 from tsuchinoko.widgets.debugmenubar import DebuggableMenuBar
+from tsuchinoko.widgets.server_editor import ServerEditor
 
 try:
     from yaml import CLoader as Loader, CDumper as Dumper, dump, load
@@ -17,7 +19,7 @@ except ImportError:
     from yaml import Loader, Dumper
 
 import zmq
-from zmq.error import ZMQError
+from zmq.error import ZMQError, Again
 import numpy as np
 from qtpy.QtGui import QIcon
 from loguru import logger
@@ -33,7 +35,7 @@ from tsuchinoko.core.messages import PauseRequest, StartRequest, GetParametersRe
     PartialDataRequest, FullDataRequest, StopRequest, Message, StateRequest, StateResponse, GetParametersResponse, \
     FullDataResponse, PartialDataResponse, MeasureRequest, \
     ConnectRequest, ConnectResponse, PushDataRequest, ExceptionResponse, PullGraphsRequest, GraphsResponse, \
-    ReplayRequest, ExitRequest
+    ReplayRequest, ExitRequest, PushGraphsRequest, SetComputeMetricsRequest
 from tsuchinoko.graphics_items.clouditem import CloudItem
 from tsuchinoko.graphics_items.indicatoritem import BetterCurveArrow
 from tsuchinoko.graphics_items.mixins import ClickRequester, request_relay, ClickRequesterPlot
@@ -56,10 +58,12 @@ class MainWindow(QMainWindow):
         open_parameters_action = QAction(self.style().standardIcon(QStyle.SP_DirOpenIcon), 'Open parameters...', parent=file_menu)
         save_data_action = QAction(self.style().standardIcon(QStyle.SP_DialogSaveButton), 'Save data as...', parent=file_menu)
         save_parameters_action = QAction(self.style().standardIcon(QStyle.SP_DialogSaveButton), 'Save parameters as...', parent=file_menu)
+        open_server_editor = QAction(self.style().standardIcon(QStyle.SP_DirOpenIcon), 'Open Server Editor', parent=file_menu)
         file_menu.addAction(open_data_action)
         file_menu.addAction(open_parameters_action)
         file_menu.addAction(save_data_action)
         file_menu.addAction(save_parameters_action)
+        file_menu.addAction(open_server_editor)
         file_menu.addAction('E&xit', self.close)
         self.setMenuBar(menubar)
 
@@ -76,6 +80,7 @@ class MainWindow(QMainWindow):
         open_data_action.triggered.connect(self.open_data)
         save_parameters_action.triggered.connect(self.save_parameters)
         open_parameters_action.triggered.connect(self.open_parameters)
+        open_server_editor.triggered.connect(self.open_server_editor)
 
         self.setWindowTitle('Tsuchinoko')
         self.setWindowIcon(QIcon(path('tsuchinoko.png')))
@@ -107,8 +112,10 @@ class MainWindow(QMainWindow):
         self.state_manager_widget.sigStart.connect(self.start)
         self.state_manager_widget.sigStop.connect(self.stop)
         self.state_manager_widget.sigReplay.connect(self.replay)
+        self.state_manager_widget.sigSetComputeMetrics.connect(self.set_compute_metrics)
         self.configuration_widget.sigPushParameter.connect(self.set_parameter)
         self.configuration_widget.sigRequestParameters.connect(self.request_parameters)
+        self.graph_manager_widget.sigPush.connect(self.push_graph)
         request_relay.sigRequestMeasure.connect(self.request_measure)
 
         self.update_thread = QThreadFutureIterator(self.update, finished_slot=self.close_zmq, name='tsuchinoko-update')
@@ -121,8 +128,8 @@ class MainWindow(QMainWindow):
         self.subscribe(self.state_manager_widget.update_state, StateResponse)
         self.subscribe(self.state_manager_widget.update_state, ConnectResponse)
         self.subscribe(self.configuration_widget.update_parameters, GetParametersResponse, invoke_as_event=True)
-        self.subscribe(self._data_callback, FullDataResponse)
-        self.subscribe(self._data_callback, PartialDataResponse)
+        self.subscribe(partial(self._data_callback, response_type='full'), FullDataResponse)
+        self.subscribe(partial(self._data_callback, response_type='partial'), PartialDataResponse)
         self.subscribe(self.refresh_state, ConnectResponse)
         self.subscribe(self.log_widget.log_exception, ExceptionResponse)
         self.subscribe(self.set_graphs, GraphsResponse, invoke_as_event=True)
@@ -163,12 +170,18 @@ class MainWindow(QMainWindow):
         self.message_queue.put(message)
         self.message_queue.put(StartRequest())
 
+    def set_compute_metrics(self, value):
+        self.message_queue.put(SetComputeMetricsRequest(value))
+
     def request_measure(self, pos):
         self.message_queue.put(MeasureRequest(pos))
 
     def request_parameters(self):
         self.message_queue.put(GetParametersRequest())
 
+    def push_graph(self, graph):
+        self.message_queue.put(PushGraphsRequest([graph]))
+
     def set_parameter(self, child_path: str, value: Any):
         if child_path:
             self.message_queue.put(SetParameterRequest(child_path, value))
@@ -209,7 +222,7 @@ class MainWindow(QMainWindow):
             try:
                 self.socket.send_pyobj(request)
                 response = self.socket.recv_pyobj()
-            except ZMQError as ex:
+            except (ZMQError, Again) as ex:
                 logger.warning(f'Unable to connect to core server at {self.core_address}...')
                 time.sleep(1)
                 # logger.exception(ex)
@@ -217,31 +230,44 @@ class MainWindow(QMainWindow):
                     self.init_socket()
                 self.data = Data()  # wipeout data and get a full update next time
                 self.last_data_size = 0
-                self.state_manager_widget.update_state(CoreState.Connecting)
+                self.state_manager_widget.update_state(CoreState.Connecting, True)
+            except UnpicklingError as ex:
+                logger.exception(ex)
+                logger.critical('The above error prevented unpacking data from the server.')
             else:
                 logger.info(f'response: {response}')
                 if not response:
                     self.get_state()
                 else:
+                    if self.state_manager_widget.state == CoreState.Connecting:
+                        logger.critical(f'Successfully connected to server at {self.core_address}.')
                     for callback, as_event in self.callbacks[type(response)]:
                         if as_event:
                             invoke_as_event(callback, *response.payload)
                         else:
                             callback(*response.payload)
 
-    def _data_callback(self, data_payload, last_data_size=None):
+    def _data_callback(self, data_payload, last_data_size=None, response_type='partial'):
         if not isinstance(data_payload, dict):  # TODO: Remove when responses are mapped to callbacks
             return
 
         if last_data_size is not None and last_data_size < len(self.data):
             raise IndexError('Overwriting of previous data prevented.')
 
-        self.data.extend(Data(**data_payload))
-        if len(data_payload['positions']):
-            invoke_as_event(self.update_graphs, self.data, self.last_data_size)
-        self.last_data_size = len(self.data)
+        if response_type == 'full':
+            self.data = Data(**data_payload)
+            self.last_data_size = 0
+        elif response_type == 'partial':
+            self.data.extend(Data(**data_payload))
+        else:
+            raise ValueError()
 
-    def refresh_state(self, _):
+        if len(data_payload['positions']):
+            # stash the length of new data early to avoid events getting confused when pausing this thread
+            old_last_data_size, self.last_data_size = self.last_data_size, len(self.data)
+            invoke_as_event(self.update_graphs, self.data, old_last_data_size)
+
+    def refresh_state(self, _, __):
         self.message_queue.queue.clear()
         self.message_queue.put(GetParametersRequest())
         self.message_queue.put(PullGraphsRequest())
@@ -344,7 +370,7 @@ class MainWindow(QMainWindow):
             self.context = None
 
     def closeEvent(self, event):
-        if not self.close_demo():
+        if not self.close_demo(confirm=True):
             event.ignore()
             return
         if self.data and len(self.data):
@@ -364,7 +390,6 @@ class MainWindow(QMainWindow):
             event.accept()
             self.close_zmq()
 
-
     def start_demo(self, demo_key):
         # If not communicating with localhost, dump connection to server
         if self.core_address != 'localhost':
@@ -372,16 +397,31 @@ class MainWindow(QMainWindow):
             self.init_socket()
 
         # if there's a child process server, exit it
-        if not self.close_demo():
+        if not self.close_demo(confirm=True):
             return
 
         suffix = Path(sys.executable).suffix 
         demo_exe = (Path(sys.executable).parent/'tsuchinoko_demo').with_suffix(suffix if suffix=='.exe' else '')
-        print(demo_exe)
+        # print(demo_exe)
         self._server = subprocess.Popen([demo_exe, demo_key])
 
-    def close_demo(self):
+    def start_server(self, path):
+        suffix = Path(sys.executable).suffix
+        bootstrap_exe = (Path(sys.executable).parent / 'tsuchinoko_bootstrap').with_suffix(suffix if suffix == '.exe' else '')
+        # print(bootstrap_exe)
+        self._server = subprocess.Popen([bootstrap_exe, path])
+
+    def close_demo(self, confirm=False):
         if self._server:
+            if confirm:
+                result = QMessageBox.question(self,
+                                              'Shutdown server?',
+                                              "A Tsuchinoko experiment server is currently running. Would you like to stop the server?",
+                                              buttons=QMessageBox.StandardButtons(
+                                                  QMessageBox.Yes | QMessageBox.Cancel),
+                                              defaultButton=QMessageBox.Yes)
+                if result != QMessageBox.Yes:
+                    return False
             self.message_queue.put(ExitRequest())
             try:
                 self._server.wait(3)
@@ -399,3 +439,9 @@ class MainWindow(QMainWindow):
                 elif result == QMessageBox.Cancel:
                     return False
         return True
+
+    stop_server = close_demo
+
+    def open_server_editor(self):
+        self.editor_window = ServerEditor(main_window=self)
+        self.editor_window.show()
